@@ -26,6 +26,8 @@
 #include <KConfig>
 #include <KConfigGroup>
 #include <QTimer>
+#include <KDirWatch>
+#include <KSharedConfig>
 
 using namespace Milou;
 
@@ -36,6 +38,15 @@ SourcesModel::SourcesModel(QObject* parent)
     m_manager = new Plasma::RunnerManager(this);
     connect(m_manager, SIGNAL(matchesChanged(QList<Plasma::QueryMatch>)),
             this, SLOT(slotMatchesChanged(QList<Plasma::QueryMatch>)));
+
+    KDirWatch* watch = KDirWatch::self();
+    connect(watch, SIGNAL(created(QString)), this, SLOT(reloadConfiguration()));
+    connect(watch, SIGNAL(dirty(QString)), this, SLOT(reloadConfiguration()));
+    watch->addFile(QStandardPaths::locate(QStandardPaths::ConfigLocation, "krunnerrc"));
+
+    m_resetTimer.setSingleShot(true);
+    m_resetTimer.setInterval(500);
+    connect(&m_resetTimer, SIGNAL(timeout()), this, SLOT(slotResetTimeout()));
 }
 
 SourcesModel::~SourcesModel()
@@ -125,6 +136,16 @@ int SourcesModel::queryLimit() const
     return m_queryLimit;
 }
 
+QString SourcesModel::runner() const
+{
+    return m_runner;
+}
+
+void SourcesModel::setRunner(const QString& runner)
+{
+    m_runner = runner;
+}
+
 void SourcesModel::setQueryLimit(int limit)
 {
     m_queryLimit = limit;
@@ -140,37 +161,55 @@ void SourcesModel::setQueryString(const QString& str)
         return;
     }
 
-    m_matches.clear();
-    m_size = 0;
-
     m_queryString = str;
+    if (m_queryString.isEmpty()) {
+        clear();
+        return;
+    }
 
-    m_manager->reset();
-    m_types.clear();
-    m_typePriority.clear();
-    m_supressSignals = true;
+    m_modelPopulated = false;
+    m_manager->setSingleModeRunnerId(m_runner);
+    m_manager->setSingleMode(!m_runner.isEmpty());
+    m_manager->launchQuery(m_queryString, m_runner);
 
-    m_manager->launchQuery(m_queryString);
-    QTimer::singleShot(250, this, SLOT(stopSuppressingSignals()));
+    // We avoid clearing the model instantly, and instead wait for the results
+    // to show up, and only then do we clear the model. In the event
+    // where there are no results, we wait for a predefined time before
+    // clearing the model
+    m_resetTimer.start();
 }
 
-void SourcesModel::stopSuppressingSignals()
+void SourcesModel::slotResetTimeout()
 {
-    m_supressSignals = false;
-
-    beginResetModel();
-    endResetModel();
+    if (!m_modelPopulated) {
+        clear();
+    }
 }
 
 //
 // Tries to make sure that all the types have the same number
 // of visible items
 //
-void SourcesModel::slotMatchesChanged(const QList<Plasma::QueryMatch>& list)
+void SourcesModel::slotMatchesChanged(const QList<Plasma::QueryMatch>& l)
 {
-    Q_FOREACH (const Plasma::QueryMatch& match, list) {
+    QList<Plasma::QueryMatch> list(l);
+    qSort(list);
+
+    QListIterator<Plasma::QueryMatch> iter(list);
+    iter.toBack();
+
+    beginResetModel();
+    m_matches.clear();
+    m_size = 0;
+    m_types.clear();
+    m_typePriority.clear();
+
+    while (iter.hasPrevious()) {
+        const Plasma::QueryMatch match = iter.previous();
         slotMatchAdded(match);
     }
+    m_modelPopulated = true;
+    endResetModel();
 }
 
 void SourcesModel::slotMatchAdded(const Plasma::QueryMatch& m)
@@ -202,10 +241,8 @@ void SourcesModel::slotMatchAdded(const Plasma::QueryMatch& m)
             m_types << matchType;
         }
 
-        if (!m_supressSignals) {
-            beginResetModel();
-            endResetModel();
-        }
+        //beginResetModel();
+        //endResetModel();
     }
 
     if (m_size == m_queryLimit) {
@@ -229,21 +266,17 @@ void SourcesModel::slotMatchAdded(const Plasma::QueryMatch& m)
         int removeRowPos = fetchRowCount(maxShownType);
         removeRowPos += m_matches[maxShownType].shown.size() - 1;
 
-        if (!m_supressSignals)
-            beginRemoveRows(QModelIndex(), removeRowPos, removeRowPos);
+        //beginRemoveRows(QModelIndex(), removeRowPos, removeRowPos);
         Plasma::QueryMatch transferMatch = m_matches[maxShownType].shown.takeLast();
         m_matches[maxShownType].hidden.append(transferMatch);
         m_size--;
-        if (!m_supressSignals)
-            endRemoveRows();
+        //endRemoveRows();
 
         int insertPos = fetchRowCount(matchType) + m_matches[matchType].shown.size();
-        if (!m_supressSignals)
-            beginInsertRows(QModelIndex(), insertPos, insertPos);
+        //beginInsertRows(QModelIndex(), insertPos, insertPos);
         m_matches[matchType].shown.append(m);
         m_size++;
-        if (!m_supressSignals)
-            endInsertRows();
+        //endInsertRows();
     }
     else {
         int pos = 0;
@@ -254,12 +287,10 @@ void SourcesModel::slotMatchAdded(const Plasma::QueryMatch& m)
             }
         }
 
-        if (!m_supressSignals)
-            beginInsertRows(QModelIndex(), pos, pos);
+        //beginInsertRows(QModelIndex(), pos, pos);
         m_matches[matchType].shown.append(m);
         m_size++;
-        if (!m_supressSignals)
-            endInsertRows();
+        //endInsertRows();
     }
 }
 
@@ -291,3 +322,9 @@ void SourcesModel::run(int index)
     Plasma::QueryMatch match = fetchMatch(index);
     m_manager->run(match);
 }
+
+void SourcesModel::reloadConfiguration()
+{
+    KSharedConfig::openConfig("krunnerrc")->reparseConfiguration();
+    m_manager->reloadConfiguration();
+}
