@@ -25,6 +25,7 @@
 
 #include <QAbstractItemModel>
 #include <QVector>
+#include <QTimer>
 
 #include <KRunner/RunnerManager>
 #include <KRunner/QueryMatch>
@@ -38,6 +39,8 @@ class MILOU_EXPORT SourcesModel : public QAbstractListModel
     Q_OBJECT
     Q_PROPERTY(QString queryString READ queryString WRITE setQueryString)
     Q_PROPERTY(int queryLimit READ queryLimit WRITE setQueryLimit)
+    Q_PROPERTY(QString runner READ runner WRITE setRunner)
+
 public:
     explicit SourcesModel(QObject* parent = 0);
     virtual ~SourcesModel();
@@ -51,6 +54,12 @@ public:
 
     virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
     virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
+    virtual bool hasChildren(const QModelIndex&) const {
+        return false;
+    }
+
+    QString runner() const;
+    void setRunner(const QString& runner);
 
     QString queryString() const;
     int queryLimit() const;
@@ -58,6 +67,7 @@ public:
     virtual QHash<int, QByteArray> roleNames() const;
 
 public slots:
+    void reloadConfiguration();
     void setQueryString(const QString& str);
     void setQueryLimit(int limit);
     void clear();
@@ -72,7 +82,7 @@ public slots:
 private slots:
     void slotMatchesChanged(const QList<Plasma::QueryMatch>& list);
     void slotMatchAdded(const Plasma::QueryMatch& match);
-    void stopSuppressingSignals();
+    void slotResetTimeout();
 
 public:
     // A list of all the types that are being shown
@@ -91,9 +101,11 @@ public:
 
     QString m_queryString;
     int m_queryLimit;
+    QString m_runner;
 
-    bool m_supressSignals;
     Plasma::RunnerManager* m_manager;
+    bool m_modelPopulated;
+    QTimer m_resetTimer;
 
     /// Returns the number of visible rows before \p type
     int fetchRowCount(const QString& type) const;
